from typing import Callable
from .html_list import tags


def atts_factory(attributes: dict) -> str:
    """Check k, v pairs and stringify them.
    If value is `True` then return just the key without the value.
    """
    return " ".join(f'{k}="{v}"' if v is not True else k for k, v in attributes.items())


## TAG BUILDER
def _el_factory(tag_name: str, end_tag: bool = True) -> Callable:
    return (
        lambda c="", a={}: _element(tag_name, c, a)
        if end_tag
        else lambda a={}: _element(tag_name, attributes=a, end_tag=end_tag)
    )


def _element(
    tag: str, content: list | str = "", attributes: dict = None, end_tag: bool = True
):
    """Build a HTML tag with attributes and content."""

    attributes = attributes or {}
    # if content is a list of HTML, join it
    content = "".join(content) if isinstance(content, list) else content

    head = f"<{tag}"
    atts = f" {atts_factory(attributes)}>" if attributes else ">"
    tail = f"{content}</{tag}>" if end_tag else ""

    return f"{head}{atts}{tail}"


# Inject HTML tag elements into Global. ☢️
for tag in tags:
    globals()[tag[0]] = _el_factory(name=tag[0], end_tag=tag[1])



tags = [
    ("a", True),
    ("abbr", True),
    ("address", True),
    ("area", False),
    ("article", True),
    ("aside", True),
    ("audio", True),
    ("base", False),
    ("blockquote", True),
    ("body", True),
    ("br", False),
    ("button", True),
    ("canvas", True),
    ("caption", True),
    ("circle", True),
    ("cite", True),
    ("code", True),
    ("col", False),
    ("colgroup", True),
    ("data", True),
    ("datalist", True),
    ("dd", True),
    ("del_", True),
    ("details", True),
    ("dialog", True),
    ("div", True),
    ("dl", True),
    ("dt", True),
    ("ellipse", True),
    ("em", True),
    ("embed", False),
    ("fieldset", True),
    ("figcaption", True),
    ("figure", True),
    ("footer", True),
    ("form", True),
    ("h1", True),
    ("h2", True),
    ("h3", True),
    ("h4", True),
    ("h5", True),
    ("h6", True),
    ("head", True),
    ("header", True),
    ("hr", False),
    ("html", True),
    ("iframe", True),
    ("img", False),
    ("input", False),
    ("kbd", True),
    ("label", True),
    ("legend", True),
    ("li", True),
    ("link", False),
    ("main", True),
    ("map", True),
    ("mark", True),
    ("menu", True),
    ("meta", False),
    ("meter", True),
    ("nav", True),
    ("noscript", True),
    ("ol", True),
    ("optgroup", True),
    ("option", True),
    ("output", True),
    ("p", True),
    ("picture", True),
    ("polygon", True),
    ("portal", True),
    ("pre", True),
    ("progress", True),
    ("param", False),
    ("q", True),
    ("rect", True),
    ("s", True),
    ("samp", True),
    ("script", True),
    ("section", True),
    ("select", True),
    ("slot", True),
    ("small", True),
    ("source", False),
    ("span", True),
    ("strong", True),
    ("style", True),
    ("sub", True),
    ("summary", True),
    ("sup", True),
    ("svg", True),
    ("table", True),
    ("tbody", True),
    ("td", True),
    ("template", True),
    ("text", True),
    ("textarea", True),
    ("tfoot", True),
    ("th", True),
    ("thead", True),
    ("time", True),
    ("title", True),
    ("tr", True),
    ("track", False),
    ("u", True),
    ("ul", True),
    ("var", True),
    ("video", True),
    ("wbr", False),
]



from .main import (
    a,
    abbr,
    address,
    area,
    article,
    aside,
    audio,
    base,
    blockquote,
    body,
    br,
    button,
    canvas,
    caption,
    circle,
    cite,
    code,
    col,
    colgroup,
    data,
    datalist,
    dd,
    del_,
    details,
    dialog,
    div,
    dl,
    dt,
    ellipse,
    em,
    embed,
    fieldset,
    figcaption,
    figure,
    footer,
    form,
    h1,
    h2,
    h3,
    h4,
    h5,
    h6,
    head,
    header,
    hr,
    html,
    iframe,
    img,
    input,
    kbd,
    label,
    legend,
    li,
    link,
    main,
    map,
    mark,
    menu,
    meta,
    meter,
    nav,
    noscript,
    ol,
    optgroup,
    option,
    output,
    p,
    picture,
    polygon,
    portal,
    pre,
    progress,
    param,
    q,
    rect,
    s,
    samp,
    script,
    section,
    select,
    slot,
    small,
    source,
    span,
    strong,
    style,
    sub,
    summary,
    sup,
    svg,
    table,
    tbody,
    td,
    template,
    text,
    textarea,
    tfoot,
    th,
    thead,
    time,
    title,
    tr,
    track,
    u,
    ul,
    var,
    video,
    wbr,
)
